##############################################################################
##############################################################################
##############################################################################




## Function to summarize difference and ratio information (D and D0)

myDfunction <- function (mydif, myrat, stat, coef) {
  
  if (stat == 1) { # linear combination of difference and ratio
    
    myDvalues = coef*mydif + (1-coef)*myrat
    
  }
  
  if (stat == 2) { # distance to origin from (ratio, difference)
    
    myDvalues = sign(mydif) * sqrt((mydif)^2 + (myrat)^2)  
    
  }
  
  myDvalues
  
}




#######################################################################



allMDbio = function (input, factor, conditions, k = 0.5, norm = "rpkm", nss = 5, lc = 1,
                      r = 10, a0per = 0.9, nclust = 15)
  
  # input:  Set of data of type Input
  
  # conditions: Levels of the factor to be compared (when the factor has more than 2 levels)
  
  # k:      When counts = 0, 0 will be changed to k. By default, k = 0.5.
  
  # norm:   Normalization method. It can be one of "rpkm" (default), "uqua"
  #         (upper quartile), "tmm" (trimmed mean of M) or "n" (no normalization).
  
  # nss:    Number of simulated samples (>= 2). By default, nss = 5.
#         If nss = 0, real samples are used to compute noise.

# lc:     Length correction in done by dividing expression by length^lc.
#         By default, lc = 1. 

# r:      Number of permutations to compute null distribution (B=5).
# a0per:  Percentile of S to compute a0. If NULL, a0 = 0. (a0per = 0.9)

{
  
  
  
  # Check if the factor introduced is already defined
  # If the factor introduced is defined and has more than 2 conditions, 
  # it will check if the conditions specified are defined too
  condition_fac = FALSE
  condition_lev = FALSE
  
  datos1 <- datos2 <- matrix()
  
  for (i in colnames(pData(input))) {
    if (factor == i) {
      condition_fac = TRUE
      if (length(levels(pData(input)[,i])) == 2) {
        
        if (!is.null(assayData(input)$exprs)) {
          datos1 <- assayData(input)$exprs[,which(pData(input)[,i] ==levels(pData(input)[,i])[1])]
          datos2 <- assayData(input)$exprs[,which(pData(input)[,i] ==levels(pData(input)[,i])[2])]
        } else {
          datos1 <- assayData(input)$counts[,which(pData(input)[,i] ==levels(pData(input)[,i])[1])]
          datos2 <- assayData(input)$counts[,which(pData(input)[,i] ==levels(pData(input)[,i])[2])]
        }
        
        # Define the comparison string
        comparison <- paste(levels(pData(input)[,i])[1], levels(pData(input)[,i])[2], sep=" - ")
        
        condition_lev = TRUE
      }
      
      else {
        if (is.null(conditions))
          stop("Error. You must specify which conditions you wish to compare when the factor has two or more conditions.\n")
        if (length(conditions) != 2)
          stop("Error. The argument conditions must contain the 2 conditions you wish to compare.")
        
        l <- conditions %in% pData(input)[,i]
        # If they are defined, they will be TRUE
        if (l[1] == TRUE && l[2] == TRUE) {
          
          if (!is.null(assayData(input)$exprs)) {
            datos1 <- assayData(input)$exprs[,which(pData(input)[,i] == conditions[1])]
            datos2 <- assayData(input)$exprs[,which(pData(input)[,i] == conditions[2])]
          } else {
            datos1 <- assayData(input)$counts[,which(pData(input)[,i] == conditions[1])]
            datos2 <- assayData(input)$counts[,which(pData(input)[,i] == conditions[2])]
          }                    
          # Define the comparison string
          comparison <- paste(conditions[1],conditions[2], sep=" - ")
          
          condition_lev = TRUE
        }
      }        
    }
  }  
  
  if (condition_fac == FALSE)
    stop("The factor you have written does not correspond with any of the ones you have defined.")
  
  if (condition_lev == FALSE)
    stop("The conditions you have written don't exist in the factor specified.\n")
  
  
  ##-------------------------------------------------------------------------##
  
  
  # Number of observations within each condition 
  n1 <- ncol(as.matrix(datos1))
  n2 <- ncol(as.matrix(datos2))
  
  # Rounding off data
  if (norm == "n") {      # no normalization
    datos1 <- round(datos1, 100)
    datos2 <- round(datos2, 100)
  }
  
  
  # Computing k
  if (is.null(k)) {
    m1 <- min(datos1[noceros(datos1, num = FALSE)], na.rm = TRUE)
    m2 <- min(datos2[noceros(datos2, num = FALSE)], na.rm = TRUE)
    k <- min(m1, m2)/2          
  } 
  
  
  # Total counts for each gene:
  suma1 <- rowSums(as.matrix(datos1))
  suma2 <- rowSums(as.matrix(datos2))
  
  # Genes with counts in any condition
  concounts <- names(which(suma1+suma2 > 0))
  
  # All genes
  todos <- rownames(as.matrix(datos1))
  
  
  # Gene length
  long <- 1000
  g.sinL <- NULL  # genes with no length defined
  
  if (!is.null(featureData(input)@data$Length)) {
    g.sinL <- names(which(is.na(featureData(input)@data$Length)))
    if (any(!is.na(featureData(input)@data$Length)) == TRUE) 
      long <- featureData(input)@data[concounts, "Length"]
  }
  
  
  # Genes with counts and with length
  gens.sin0 <- setdiff(concounts, g.sinL)
  
  
  # cond1 and cond2 in the same matrix
  datitos <- cbind(datos1, datos2)
  datitos <- datitos[concounts,]  # selecting only genes with counts
  
  
  
  #-------------------------------------------------------------------------#
  
  
  ## Simulated replicates
  
  simu = FALSE  # By default, there are replicates and they are not simulated
  
  if ((n1+n2) <= 2) {   
    
    simu = TRUE  # Replicates need to be simulated
    
    if (nss == 0) { nss <- 5 }
    
    datos.sim <- sim.samples(counts1 = suma1, counts2 = suma2,
                             pnr = 1, nss = nss, v = 0.02)
    
    # including also the available replicate
    datos.sim[[1]] = cbind(datos.sim[[1]], suma1)
    datos.sim[[2]] = cbind(datos.sim[[2]], suma2)
    
    n1 = n2 = nss + 1
    
    datitos <- cbind(datos.sim[[1]], datos.sim[[2]]) # cond1 and cond2 in the same matrix
    datitos = datitos[concounts,]
    
  }
  
  
  
  #-------------------------------------------------------------------------#
  
  
  
  ## Normalization  
  
  if (norm == "n") {       # no normalization
    datitos.0 <- sinceros(datitos, k = k)
    datitos.norm <- datitos.0[gens.sin0, ]
  }
  
  if (norm == "rpkm") {      # RPKM
    datitos.0 <- rpkm(datitos, long = long, k = k, lc = lc)
    datitos.norm <- datitos.0[gens.sin0, ]
  }
  
  if (norm == "uqua") {      # Upper Quartile
    datitos.0 <- uqua(datitos, long = long, lc = lc, k = k)
    datitos.norm <- datitos.0[gens.sin0, ] 
  }
  
  if (norm == "tmm") {
    datitos.0 <- tmm(datitos, long = long, lc = lc, k = k)
    datitos.norm <- datitos.0[gens.sin0, ]       
  }
  
  datos1.norm <- datitos.norm[ ,1:n1]
  datos2.norm <- datitos.norm[ ,(n1+1):(n1+n2)]
  
  
  
  
  
  #-------------------------------------------------------------------------#
  
  
  
  ## Noise distribution
  
  Zr = NULL
  
  if (n1+n2 <= 8) {  # excluding the identity permutation
    
    Zr = share.info(mydata = datitos.norm, n1 = n1, n2 = n2, r = r, nclust = nclust)
 
    
  } else {   # r permutations
    
    for (i in 1:r) {
      
      print(paste("r =", i))
      
      mipermu = sample(1:(n1+n2))
            
      mipermu = datitos.norm[,mipermu]
      
      mean1 = rowMeans(mipermu[,1:n1])
      mean2 = rowMeans(mipermu[,(n1+1):(n1+n2)])        
      
      sd1 = apply(mipermu[,1:n1], 1, sd)
      sd2 = apply(mipermu[,(n1+1):(n1+n2)], 1, sd)
      
      myparam = list("n" = c(n1,n2), "sd" = cbind(sd1,sd2))
      
      MDperm <- MDbio(dat = cbind(mean1, mean2), param = myparam, a0per = a0per)
      
      Zr = cbind(Zr, myDfunction(mydif = MDperm$D, myrat = MDperm$M, stat = 1, coef = 0.5))
       
    }
  }
  
    
  
  
  #-------------------------------------------------------------------------#
  
  
  ## Z-score for different experimental conditions (SIGNAL)
  
  mean1 = rowMeans(as.matrix(datos1.norm))
  mean2 = rowMeans(as.matrix(datos2.norm))        
  
  sd1 = apply(as.matrix(datos1.norm), 1, sd)
  sd2 = apply(as.matrix(datos2.norm), 1, sd)
  
  myparam = list("n" = c(n1,n2), "sd" = cbind(sd1,sd2))
  
  MDs <- MDbio(dat = cbind(mean1, mean2), param = myparam, a0per = a0per)
  
  
  Zs = myDfunction(mydif = MDs$D, myrat = MDs$M, stat = 1, coef = 0.5)
    
  
  
    
  
  #-------------------------------------------------------------------------#
  
  
  
  ## Completing M and D (in signal)
  
  lev1 <- mean1[todos]
  lev2 <- mean2[todos]
  names(lev1) <- names(lev2) <- todos
  
  Zs <- as.numeric(Zs)
  names(Zs) <- rownames(MDs$M)
  
  Zs <- Zs[todos]
  names(Zs) <- todos
  
  
  ## Computing Zn
  Zn = as.numeric(Zr)
  
  
  
  #-------------------------------------------------------------------------#
  
  ## Results
  list("k" = k, "comp" = comparison, "Level1" = lev1, "Level2" = lev2, "Zs" = Zs, "Zn" = Zn)
  
}





