#############################################################################

## NOISeqBIO


noiseqbio = function (input, k = 0.5, norm = c("rpkm","uqua","tmm","n"), nclust = 15,
                      factor=NULL, conditions = NULL, nss = 5, lc = 1, r = 50, 
                      a0per = 0.9, random.seed = 12345) 
  
  # input: Object containing gene counts and as many columns as samples.
  
  # k:      When counts = 0, 0 will be changed to k. By default, k = 0.5.
  
  # norm:   Normalization method. It can be one of "rpkm" (default), "uqua"
  #         (upper quartile), "tmm" (trimmed mean of M) or "n" (no normalization).
  
  # factor: String with the factor to choose which conditions you are going
  #         to compare.
  
# conditions: String with the conditions to compare in case one factor contains more
#         than 2 conditions.

# nss:    Number of simulated samples (>= 2). By default, nss = 5.
#         If nss = 0, real samples are used to compute noise.

# lc:     Length correction in done by dividing expression by length^lc.
#         By default, lc = 1. 


# r:      Number of permutations to compute null distribution (r=10).
# a0per:  Percentile of S to compute a0. If NULL, a0 = 0. (a0per = 0.9)




{
  
  if (inherits(input,"eSet") == FALSE)
    stop("Error. You must give an object generated by the readData function\n")
  
  if (is.null(factor))
    stop("Error. You must specify the factor to know which conditions you wish to compare.
         Please, give the argument 'factor'.\n")
  
  norm <- match.arg(norm)
  
  
  # Random seed
  if (!is.null(random.seed)) set.seed(random.seed)  
  
  
  
  # Z-scores for signal and noise
  print("Computing Z values...")
  miMD <- allMDbio(input, factor, k = k, norm = norm, conditions = conditions, nss = nss, lc = lc, 
                    r = r, a0per = a0per, nclust = nclust)
  
  
  
  
  #------------------------------------------------------------------#
  
  
  
  ## Probability of differential expression
  
  print("Computing probability of differential expression...")
  
  
  # 5) Using KDE estimators of f0 and f
    
    desde <- min(c(miMD$Zs,miMD$Zn), na.rm = TRUE)
    hasta <- max(c(miMD$Zs,miMD$Zn), na.rm = TRUE)
    
    fdens <- density(miMD$Zs, adjust = 1.5, n = 5000, from = desde, to = hasta, na.rm = TRUE)
    f <- approxfun(fdens)
    
    f0dens <- density(miMD$Zn, adjust = 1.5, n = 5000, from = desde, to = hasta, na.rm = TRUE)
    f0 <- approxfun(f0dens)
    
    if (f0(0)/f(0) < 1) print("WARNING: f0(0)/f(0) < 1 => FP with Z~0 will be detected.")
    
    f0.f <- f0(miMD$Zs) / f(miMD$Zs)
      
  
  
  ## 6) ESTIMATION of p0
  
    p0 <- min(1/f0.f, na.rm = TRUE)
    p0 = min(p0,1)

  print(paste("p0 =", p0))
    
  
  ## 7) PROBABILITY of DIFFERENTIAL EXPRESSION
  myprob <- 1 - p0*f0.f
  if(min(myprob, na.rm = TRUE) < 0) {
    print(summary(f0.f))
  } 
  names(myprob) <- names(miMD$Zs)
  
  print("Probability")
  print(summary(myprob))
       
  
  if (!is.null(assayData(input)$exprs))
    todos <- rownames(as.matrix(assayData(input)$exprs))
  else
    todos <- rownames(as.matrix(assayData(input)$counts))
  
  myprob <- myprob[todos]
  names(myprob) <- todos
  
  ## Results
  resultat <- data.frame("level_1" = miMD$Level1, "level_2" = miMD$Level2, "theta" = miMD$Zs, "prob" = myprob)
  rownames(resultat) <- todos
  
  colnames(resultat)[1] <- unlist(strsplit(miMD$comp," "))[1]
  colnames(resultat)[2] <- unlist(strsplit(miMD$comp," "))[3]
  
  #   resultat <- data.frame(resultat, "ranking" = ranking(resultat)$statistic)
  if (!is.null(featureData(input)@data$Length))
    resultat <- data.frame(resultat, "length" = as.numeric(as.character(featureData(input)@data[todos,"Length"])))
  if (!is.null(featureData(input)@data$Chromosome))
    resultat <- data.frame(resultat, "Chrom" = as.numeric(as.character(featureData(input)@data$Chromosome)),
                           "GeneStart" = as.numeric(as.character(featureData(input)@data$GeneStart)),
                           "GeneEnd" = as.numeric(as.character(featureData(input)@data$GeneEnd)))
  
  Output(data = list(resultat), method=norm, k=miMD$k, lc=lc, factor=factor, nss=nss,
         comparison=miMD$comp)
}



